@@ -283,7 +283,7 @@ def add_ports_to_network(task, network_uuid, security_groups=None):
                         "the following ports: %(ports)s.",
                         {'node': node.uuid, 'ports': failures})
     else:
-        LOG.info('For node %(node_uuid)s in network %(net)s, successfully ',
+        LOG.info('For node %(node_uuid)s in network %(net)s, successfully '
                  'created ports (ironic ID: neutron ID): %(ports)s.',
                  {'node_uuid': node.uuid, 'net': network_uuid, 'ports': ports})
 
@@ -340,6 +340,9 @@ def remove_neutron_ports(task, params):
 
         try:
             client.delete_port(port['id'])
+        # NOTE(mgoddard): Ignore if the port was deleted by nova.
+        except neutron_exceptions.PortNotFoundClient:
+            LOG.info('Port %s was not found while deleting.', port['id'])
         except neutron_exceptions.NeutronClientException as e:
             msg = (_('Could not remove VIF %(vif)s of node %(node)s, possibly '
                      'a network issue: %(exc)s') %
