@@ -346,6 +346,16 @@ def cleaning_error_handler(task, msg, tear_down_cleaning=True,
                            set_fail_state=True):
     """Put a failed node in CLEANFAIL and maintenance."""
     node = task.node
+
+    if tear_down_cleaning:
+        try:
+            task.driver.deploy.tear_down_cleaning(task)
+        except Exception as e:
+            msg2 = ('Failed to tear down cleaning on node %(uuid)s, '
+                    'reason: %(err)s' % {'err': e, 'uuid': node.uuid})
+            LOG.exception(msg2)
+            msg = _('%s. Also failed to tear down cleaning.') % msg
+
     if node.provision_state in (
             states.CLEANING,
             states.CLEANWAIT,
@@ -364,19 +374,33 @@ def cleaning_error_handler(task, msg, tear_down_cleaning=True,
     node.maintenance = True
     node.maintenance_reason = msg
     node.save()
-    if tear_down_cleaning:
-        try:
-            task.driver.deploy.tear_down_cleaning(task)
-        except Exception as e:
-            msg = ('Failed to tear down cleaning on node %(uuid)s, '
-                   'reason: %(err)s' % {'err': e, 'uuid': node.uuid})
-            LOG.exception(msg)
 
     if set_fail_state:
         target_state = states.MANAGEABLE if manual_clean else None
         task.process_event('fail', target_state=target_state)
 
 
+@task_manager.require_exclusive_lock
+def abort_on_conductor_take_over(task):
+    """Set node's state when a task was aborted due to conductor take over.
+
+    :param task: a TaskManager instance.
+    """
+    msg = _('Operation was aborted due to conductor take over')
+    # By this time the "fail" even was processed, so we cannot end up in
+    # CLEANING or CLEAN WAIT, only in CLEAN FAIL.
+    if task.node.provision_state == states.CLEANFAIL:
+        cleaning_error_handler(task, msg, set_fail_state=False)
+    else:
+        # For aborted deployment (and potentially other operations), just set
+        # the last_error accordingly.
+        task.node.last_error = msg
+        task.node.save()
+
+    LOG.warning('Aborted the current operation on node %s due to '
+                'conductor take over', task.node.uuid)
+
+
 def rescuing_error_handler(task, msg, set_fail_state=True):
     """Cleanup rescue task after timeout or failure.
 
