@@ -583,6 +583,8 @@ class NeutronVIFPortIDMixin(VIFPortIDMixin):
 
         self._clear_vif_from_port_like_obj(port_like_obj)
 
-        # NOTE(vsaienko) allow to unplug VIFs from ACTIVE instance.
-        if task.node.provision_state == states.ACTIVE:
+        # NOTE(vsaienko): allow to unplug VIFs from ACTIVE instance.
+        # NOTE(TheJulia): Also ensure that we delete the vif when in
+        # DELETING state.
+        if task.node.provision_state in [states.ACTIVE, states.DELETING]:
             neutron.unbind_neutron_port(vif_id, context=task.context)
