@@ -19,6 +19,7 @@ from oslo_config import cfg
 
 from ironic.common import dhcp_factory
 from ironic.common import exception
+from ironic.common import image_service
 from ironic.common import images
 from ironic.common import raid
 from ironic.common import states
@@ -31,6 +32,7 @@ from ironic.drivers.modules import agent_client
 from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules import fake
 from ironic.drivers.modules.network import flat as flat_network
+from ironic.drivers.modules.network import neutron as neutron_network
 from ironic.drivers.modules import pxe
 from ironic.drivers.modules.storage import noop as noop_storage
 from ironic.drivers import utils as driver_utils
@@ -342,8 +344,11 @@ class TestAgentDeploy(db_base.DbTestCase):
     @mock.patch.object(flat_network.FlatNetwork,
                        'unconfigure_tenant_networks',
                        spec_set=True, autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork, 'validate',
+                       spec_set=True, autospec=True)
     def test_prepare(
-            self, unconfigure_tenant_net_mock, add_provisioning_net_mock,
+            self, validate_net_mock,
+            unconfigure_tenant_net_mock, add_provisioning_net_mock,
             build_instance_info_mock, build_options_mock,
             pxe_prepare_ramdisk_mock, storage_driver_info_mock,
             storage_attach_volumes_mock):
@@ -352,30 +357,215 @@ class TestAgentDeploy(db_base.DbTestCase):
             task.node.provision_state = states.DEPLOYING
             build_instance_info_mock.return_value = {'foo': 'bar'}
             build_options_mock.return_value = {'a': 'b'}
-
             self.driver.prepare(task)
-
+            storage_driver_info_mock.assert_called_once_with(task)
+            validate_net_mock.assert_called_once_with(mock.ANY, task)
+            add_provisioning_net_mock.assert_called_once_with(mock.ANY, task)
+            unconfigure_tenant_net_mock.assert_called_once_with(mock.ANY, task)
+            storage_attach_volumes_mock.assert_called_once_with(
+                task.driver.storage, task)
             build_instance_info_mock.assert_called_once_with(task)
             build_options_mock.assert_called_once_with(task.node)
             pxe_prepare_ramdisk_mock.assert_called_once_with(
                 task, {'a': 'b'})
-            add_provisioning_net_mock.assert_called_once_with(mock.ANY, task)
-            unconfigure_tenant_net_mock.assert_called_once_with(mock.ANY, task)
-            storage_driver_info_mock.assert_called_once_with(task)
-            storage_attach_volumes_mock.assert_called_once_with(
-                task.driver.storage, task)
-
         self.node.refresh()
         self.assertEqual('bar', self.node.instance_info['foo'])
 
+    @mock.patch.object(noop_storage.NoopStorage, 'attach_volumes',
+                       autospec=True)
+    @mock.patch.object(deploy_utils, 'populate_storage_driver_internal_info')
+    @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk')
+    @mock.patch.object(deploy_utils, 'build_agent_options')
+    @mock.patch.object(deploy_utils, 'build_instance_info_for_deploy')
+    @mock.patch.object(neutron_network.NeutronNetwork,
+                       'add_provisioning_network',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(neutron_network.NeutronNetwork,
+                       'unconfigure_tenant_networks',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(neutron_network.NeutronNetwork, 'validate',
+                       spec_set=True, autospec=True)
+    def test_prepare_with_neutron_net(
+            self, validate_net_mock,
+            unconfigure_tenant_net_mock, add_provisioning_net_mock,
+            build_instance_info_mock, build_options_mock,
+            pxe_prepare_ramdisk_mock, storage_driver_info_mock,
+            storage_attach_volumes_mock):
+        node = self.node
+        node.network_interface = 'neutron'
+        node.save()
+        with task_manager.acquire(
+                self.context, self.node['uuid'], shared=False) as task:
+            task.node.provision_state = states.DEPLOYING
+            build_instance_info_mock.return_value = {'foo': 'bar'}
+            build_options_mock.return_value = {'a': 'b'}
+            self.driver.prepare(task)
+            storage_driver_info_mock.assert_called_once_with(task)
+            validate_net_mock.assert_called_once_with(mock.ANY, task)
+            add_provisioning_net_mock.assert_called_once_with(mock.ANY, task)
+            unconfigure_tenant_net_mock.assert_called_once_with(mock.ANY, task)
+            storage_attach_volumes_mock.assert_called_once_with(
+                task.driver.storage, task)
+            build_instance_info_mock.assert_called_once_with(task)
+            build_options_mock.assert_called_once_with(task.node)
+            pxe_prepare_ramdisk_mock.assert_called_once_with(
+                task, {'a': 'b'})
+        self.node.refresh()
+        self.assertEqual('bar', self.node.instance_info['foo'])
+
+    @mock.patch.object(noop_storage.NoopStorage, 'attach_volumes',
+                       autospec=True)
+    @mock.patch.object(deploy_utils, 'populate_storage_driver_internal_info')
+    @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk')
+    @mock.patch.object(deploy_utils, 'build_agent_options')
+    @mock.patch.object(image_service.HttpImageService, 'validate_href',
+                       autospec=True)
+    @mock.patch.object(neutron_network.NeutronNetwork,
+                       'add_provisioning_network',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(neutron_network.NeutronNetwork,
+                       'unconfigure_tenant_networks',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(neutron_network.NeutronNetwork, 'validate',
+                       spec_set=True, autospec=True)
+    def test_prepare_with_neutron_net_exc_no_capabilities(
+            self, validate_net_mock,
+            unconfigure_tenant_net_mock, add_provisioning_net_mock,
+            validate_href_mock, build_options_mock,
+            pxe_prepare_ramdisk_mock, storage_driver_info_mock,
+            storage_attach_volumes_mock):
+        node = self.node
+        node.network_interface = 'neutron'
+        node.save()
+        validate_net_mock.side_effect = [
+            exception.InvalidParameterValue('invalid'), None]
+        with task_manager.acquire(
+                self.context, self.node['uuid'], shared=False) as task:
+            task.node.provision_state = states.DEPLOYING
+            build_options_mock.return_value = {'a': 'b'}
+            self.driver.prepare(task)
+            storage_driver_info_mock.assert_called_once_with(task)
+            self.assertEqual(2, validate_net_mock.call_count)
+            add_provisioning_net_mock.assert_called_once_with(mock.ANY, task)
+            unconfigure_tenant_net_mock.assert_called_once_with(mock.ANY, task)
+            storage_attach_volumes_mock.assert_called_once_with(
+                task.driver.storage, task)
+            validate_href_mock.assert_called_once_with(mock.ANY, 'fake-image',
+                                                       secret=False)
+            build_options_mock.assert_called_once_with(task.node)
+            pxe_prepare_ramdisk_mock.assert_called_once_with(
+                task, {'a': 'b'})
+        self.node.refresh()
+        capabilities = self.node.instance_info['capabilities']
+        self.assertEqual('local', capabilities['boot_option'])
+
+    @mock.patch.object(noop_storage.NoopStorage, 'attach_volumes',
+                       autospec=True)
+    @mock.patch.object(deploy_utils, 'populate_storage_driver_internal_info')
+    @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk')
+    @mock.patch.object(deploy_utils, 'build_agent_options')
+    @mock.patch.object(image_service.HttpImageService, 'validate_href',
+                       autospec=True)
+    @mock.patch.object(neutron_network.NeutronNetwork,
+                       'add_provisioning_network',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(neutron_network.NeutronNetwork,
+                       'unconfigure_tenant_networks',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(neutron_network.NeutronNetwork, 'validate',
+                       spec_set=True, autospec=True)
+    def test_prepare_with_neutron_net_exc_no_capabilities_overwrite(
+            self, validate_net_mock,
+            unconfigure_tenant_net_mock, add_provisioning_net_mock,
+            validate_href_mock, build_options_mock,
+            pxe_prepare_ramdisk_mock, storage_driver_info_mock,
+            storage_attach_volumes_mock):
+        node = self.node
+        node.network_interface = 'neutron'
+        instance_info = node.instance_info
+        instance_info['capabilities'] = {"cat": "meow"}
+        node.instance_info = instance_info
+        node.save()
+        validate_net_mock.side_effect = [
+            exception.InvalidParameterValue('invalid'), None]
+        with task_manager.acquire(
+                self.context, self.node['uuid'], shared=False) as task:
+            task.node.provision_state = states.DEPLOYING
+            build_options_mock.return_value = {'a': 'b'}
+            self.driver.prepare(task)
+            storage_driver_info_mock.assert_called_once_with(task)
+            self.assertEqual(2, validate_net_mock.call_count)
+            add_provisioning_net_mock.assert_called_once_with(mock.ANY, task)
+            unconfigure_tenant_net_mock.assert_called_once_with(mock.ANY, task)
+            storage_attach_volumes_mock.assert_called_once_with(
+                task.driver.storage, task)
+            validate_href_mock.assert_called_once_with(mock.ANY, 'fake-image',
+                                                       secret=False)
+            build_options_mock.assert_called_once_with(task.node)
+            pxe_prepare_ramdisk_mock.assert_called_once_with(
+                task, {'a': 'b'})
+        self.node.refresh()
+        capabilities = self.node.instance_info['capabilities']
+        self.assertEqual('local', capabilities['boot_option'])
+        self.assertEqual('meow', capabilities['cat'])
+
+    @mock.patch.object(noop_storage.NoopStorage, 'attach_volumes',
+                       autospec=True)
+    @mock.patch.object(deploy_utils, 'populate_storage_driver_internal_info')
+    @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk')
+    @mock.patch.object(deploy_utils, 'build_agent_options')
+    @mock.patch.object(deploy_utils, 'build_instance_info_for_deploy')
+    @mock.patch.object(neutron_network.NeutronNetwork,
+                       'add_provisioning_network',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(neutron_network.NeutronNetwork,
+                       'unconfigure_tenant_networks',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(neutron_network.NeutronNetwork, 'validate',
+                       spec_set=True, autospec=True)
+    def test_prepare_with_neutron_net_exc_reraise(
+            self, validate_net_mock,
+            unconfigure_tenant_net_mock, add_provisioning_net_mock,
+            build_instance_info_mock, build_options_mock,
+            pxe_prepare_ramdisk_mock, storage_driver_info_mock,
+            storage_attach_volumes_mock):
+        node = self.node
+        node.network_interface = 'neutron'
+        instance_info = node.instance_info
+        instance_info['capabilities'] = {"boot_option": "netboot"}
+        node.instance_info = instance_info
+        node.save()
+        validate_net_mock.side_effect = (
+            exception.InvalidParameterValue('invalid'))
+        with task_manager.acquire(
+                self.context, self.node['uuid'], shared=False) as task:
+            task.node.provision_state = states.DEPLOYING
+            self.assertRaises(exception.InvalidParameterValue,
+                              task.driver.deploy.prepare,
+                              task)
+            storage_driver_info_mock.assert_called_once_with(task)
+            validate_net_mock.assert_called_once_with(mock.ANY, task)
+            self.assertFalse(add_provisioning_net_mock.called)
+            self.assertFalse(unconfigure_tenant_net_mock.called)
+            self.assertFalse(storage_attach_volumes_mock.called)
+            self.assertFalse(build_instance_info_mock.called)
+            self.assertFalse(build_options_mock.called)
+            self.assertFalse(pxe_prepare_ramdisk_mock.called)
+        self.node.refresh()
+        capabilities = self.node.instance_info['capabilities']
+        self.assertEqual('netboot', capabilities['boot_option'])
+
     @mock.patch.object(flat_network.FlatNetwork, 'add_provisioning_network',
                        spec_set=True, autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork, 'validate',
+                       spec_set=True, autospec=True)
     @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk')
     @mock.patch.object(deploy_utils, 'build_agent_options')
     @mock.patch.object(deploy_utils, 'build_instance_info_for_deploy')
     def test_prepare_manage_agent_boot_false(
-            self, build_instance_info_mock, build_options_mock,
-            pxe_prepare_ramdisk_mock, add_provisioning_net_mock):
+            self, build_instance_info_mock,
+            build_options_mock, pxe_prepare_ramdisk_mock,
+            validate_net_mock, add_provisioning_net_mock):
         self.config(group='agent', manage_agent_boot=False)
         with task_manager.acquire(
                 self.context, self.node['uuid'], shared=False) as task:
@@ -384,6 +574,7 @@ class TestAgentDeploy(db_base.DbTestCase):
 
             self.driver.prepare(task)
 
+            validate_net_mock.assert_called_once_with(mock.ANY, task)
             build_instance_info_mock.assert_called_once_with(task)
             add_provisioning_net_mock.assert_called_once_with(mock.ANY, task)
             self.assertFalse(build_options_mock.called)
@@ -461,6 +652,8 @@ class TestAgentDeploy(db_base.DbTestCase):
     @mock.patch.object(flat_network.FlatNetwork,
                        'unconfigure_tenant_networks',
                        spec_set=True, autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork, 'validate',
+                       spec_set=True, autospec=True)
     @mock.patch.object(pxe.PXEBoot, 'prepare_instance', autospec=True)
     @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk', autospec=True)
     @mock.patch.object(deploy_utils, 'build_agent_options', autospec=True)
@@ -469,16 +662,16 @@ class TestAgentDeploy(db_base.DbTestCase):
     def test_prepare_storage_write_false(
             self, build_instance_info_mock, build_options_mock,
             pxe_prepare_ramdisk_mock, pxe_prepare_instance_mock,
-            remove_tenant_net_mock, add_provisioning_net_mock,
-            storage_driver_info_mock, storage_attach_volumes_mock,
-            should_write_image_mock):
+            validate_net_mock, remove_tenant_net_mock,
+            add_provisioning_net_mock, storage_driver_info_mock,
+            storage_attach_volumes_mock, should_write_image_mock):
         should_write_image_mock.return_value = False
         with task_manager.acquire(
                 self.context, self.node['uuid'], shared=False) as task:
             task.node.provision_state = states.DEPLOYING
 
             self.driver.prepare(task)
-
+            validate_net_mock.assert_called_once_with(mock.ANY, task)
             self.assertFalse(build_instance_info_mock.called)
             self.assertFalse(build_options_mock.called)
             self.assertFalse(pxe_prepare_ramdisk_mock.called)
@@ -509,6 +702,8 @@ class TestAgentDeploy(db_base.DbTestCase):
 
     @mock.patch.object(flat_network.FlatNetwork, 'add_provisioning_network',
                        spec_set=True, autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork, 'validate',
+                       spec_set=True, autospec=True)
     @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk')
     @mock.patch.object(deploy_utils, 'build_agent_options')
     @mock.patch.object(deploy_utils, 'build_instance_info_for_deploy')
@@ -518,6 +713,7 @@ class TestAgentDeploy(db_base.DbTestCase):
                                       build_instance_info_mock,
                                       build_options_mock,
                                       pxe_prepare_ramdisk_mock,
+                                      validate_net_mock,
                                       add_provisioning_net_mock):
         mock_write.return_value = False
         with task_manager.acquire(
@@ -527,6 +723,7 @@ class TestAgentDeploy(db_base.DbTestCase):
             build_options_mock.return_value = {'a': 'b'}
 
             self.driver.prepare(task)
+            validate_net_mock.assert_called_once_with(mock.ANY, task)
             build_instance_info_mock.assert_not_called()
             build_options_mock.assert_not_called()
             pxe_prepare_ramdisk_mock.assert_not_called()
@@ -754,6 +951,7 @@ class TestAgentDeploy(db_base.DbTestCase):
             self.assertEqual(states.ACTIVE,
                              task.node.target_provision_state)
 
+    @mock.patch.object(agent.LOG, 'warning', spec_set=True, autospec=True)
     @mock.patch.object(agent.AgentDeployMixin, '_get_uuid_from_result',
                        autospec=True)
     @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
@@ -761,17 +959,17 @@ class TestAgentDeploy(db_base.DbTestCase):
                        spec=types.FunctionType)
     @mock.patch.object(agent_client.AgentClient, 'power_off',
                        spec=types.FunctionType)
-    @mock.patch.object(pxe.PXEBoot, 'prepare_instance',
+    @mock.patch.object(agent.AgentDeployMixin, 'prepare_instance_to_boot',
                        autospec=True)
     @mock.patch('ironic.drivers.modules.agent.AgentDeployMixin'
                 '.check_deploy_success', autospec=True)
-    @mock.patch.object(pxe.PXEBoot, 'clean_up_ramdisk', autospec=True)
-    def test_reboot_to_instance(self, clean_pxe_mock, check_deploy_mock,
-                                prepare_mock, power_off_mock,
+    def test_reboot_to_instance(self, check_deploy_mock,
+                                prepare_instance_mock, power_off_mock,
                                 get_power_state_mock, node_power_action_mock,
-                                uuid_mock):
+                                uuid_mock, log_mock):
+        self.config(manage_agent_boot=True, group='agent')
         check_deploy_mock.return_value = None
-        uuid_mock.return_value = 'root_uuid'
+        uuid_mock.return_value = None
         self.node.provision_state = states.DEPLOYWAIT
         self.node.target_provision_state = states.ACTIVE
         self.node.save()
@@ -779,22 +977,69 @@ class TestAgentDeploy(db_base.DbTestCase):
                                   shared=False) as task:
             get_power_state_mock.return_value = states.POWER_OFF
             task.node.driver_internal_info['is_whole_disk_image'] = True
-
             task.driver.deploy.reboot_to_instance(task)
-
-            clean_pxe_mock.assert_called_once_with(task.driver.boot, task)
             check_deploy_mock.assert_called_once_with(mock.ANY, task.node)
+            uuid_mock.assert_called_once_with(mock.ANY, task, 'root_uuid')
+            self.assertNotIn('root_uuid_or_disk_id',
+                             task.node.driver_internal_info)
+            self.assertTrue(log_mock.called)
+            self.assertIn("Ironic Python Agent version 3.1.0 and beyond",
+                          log_mock.call_args[0][0])
+            prepare_instance_mock.assert_called_once_with(mock.ANY, task,
+                                                          None, None)
             power_off_mock.assert_called_once_with(task.node)
             get_power_state_mock.assert_called_once_with(task)
             node_power_action_mock.assert_called_once_with(
                 task, states.POWER_ON)
-            self.assertFalse(prepare_mock.called)
             self.assertEqual(states.ACTIVE, task.node.provision_state)
             self.assertEqual(states.NOSTATE, task.node.target_provision_state)
+
+    @mock.patch.object(agent.LOG, 'warning', spec_set=True, autospec=True)
+    @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
+    @mock.patch.object(agent.AgentDeployMixin, '_get_uuid_from_result',
+                       autospec=True)
+    @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
+    @mock.patch.object(fake.FakePower, 'get_power_state',
+                       spec=types.FunctionType)
+    @mock.patch.object(agent_client.AgentClient, 'power_off',
+                       spec=types.FunctionType)
+    @mock.patch.object(agent.AgentDeployMixin, 'prepare_instance_to_boot',
+                       autospec=True)
+    @mock.patch('ironic.drivers.modules.agent.AgentDeployMixin'
+                '.check_deploy_success', autospec=True)
+    def test_reboot_to_instance_no_manage_agent_boot(self, check_deploy_mock,
+                                                     prepare_instance_mock,
+                                                     power_off_mock,
+                                                     get_power_state_mock,
+                                                     node_power_action_mock,
+                                                     uuid_mock, bootdev_mock,
+                                                     log_mock):
+        self.config(manage_agent_boot=False, group='agent')
+        check_deploy_mock.return_value = None
+        uuid_mock.return_value = None
+        self.node.provision_state = states.DEPLOYWAIT
+        self.node.target_provision_state = states.ACTIVE
+        self.node.save()
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            get_power_state_mock.return_value = states.POWER_OFF
+            task.node.driver_internal_info['is_whole_disk_image'] = True
+            task.driver.deploy.reboot_to_instance(task)
+            check_deploy_mock.assert_called_once_with(mock.ANY, task.node)
+            uuid_mock.assert_called_once_with(mock.ANY, task, 'root_uuid')
             self.assertNotIn('root_uuid_or_disk_id',
                              task.node.driver_internal_info)
-            self.assertFalse(uuid_mock.called)
+            self.assertFalse(log_mock.called)
+            self.assertFalse(prepare_instance_mock.called)
+            bootdev_mock.assert_called_once_with(task, 'disk', persistent=True)
+            power_off_mock.assert_called_once_with(task.node)
+            get_power_state_mock.assert_called_once_with(task)
+            node_power_action_mock.assert_called_once_with(
+                task, states.POWER_ON)
+            self.assertEqual(states.ACTIVE, task.node.provision_state)
+            self.assertEqual(states.NOSTATE, task.node.target_provision_state)
 
+    @mock.patch.object(agent.LOG, 'warning', spec_set=True, autospec=True)
     @mock.patch.object(deploy_utils, 'get_boot_mode_for_deploy', autospec=True)
     @mock.patch.object(agent.AgentDeployMixin, '_get_uuid_from_result',
                        autospec=True)
@@ -803,17 +1048,17 @@ class TestAgentDeploy(db_base.DbTestCase):
                        spec=types.FunctionType)
     @mock.patch.object(agent_client.AgentClient, 'power_off',
                        spec=types.FunctionType)
-    @mock.patch.object(pxe.PXEBoot, 'prepare_instance',
+    @mock.patch.object(agent.AgentDeployMixin, 'prepare_instance_to_boot',
                        autospec=True)
     @mock.patch('ironic.drivers.modules.agent.AgentDeployMixin'
                 '.check_deploy_success', autospec=True)
-    @mock.patch.object(pxe.PXEBoot, 'clean_up_ramdisk', autospec=True)
-    def test_reboot_to_instance_partition_image(self, clean_pxe_mock,
-                                                check_deploy_mock,
-                                                prepare_mock, power_off_mock,
+    def test_reboot_to_instance_partition_image(self, check_deploy_mock,
+                                                prepare_instance_mock,
+                                                power_off_mock,
                                                 get_power_state_mock,
                                                 node_power_action_mock,
-                                                uuid_mock, boot_mode_mock):
+                                                uuid_mock, boot_mode_mock,
+                                                log_mock):
         check_deploy_mock.return_value = None
         uuid_mock.return_value = 'root_uuid'
         self.node.provision_state = states.DEPLOYWAIT
@@ -826,71 +1071,25 @@ class TestAgentDeploy(db_base.DbTestCase):
             driver_internal_info = task.node.driver_internal_info
             driver_internal_info['is_whole_disk_image'] = False
             task.node.driver_internal_info = driver_internal_info
-
             task.driver.deploy.reboot_to_instance(task)
-
-            self.assertFalse(clean_pxe_mock.called)
             check_deploy_mock.assert_called_once_with(mock.ANY, task.node)
-            power_off_mock.assert_called_once_with(task.node)
-            get_power_state_mock.assert_called_once_with(task)
-            node_power_action_mock.assert_called_once_with(
-                task, states.POWER_ON)
-            prepare_mock.assert_called_once_with(task.driver.boot, task)
-            self.assertEqual(states.ACTIVE, task.node.provision_state)
-            self.assertEqual(states.NOSTATE, task.node.target_provision_state)
+            uuid_mock.assert_called_once_with(mock.ANY,
+                                              task, 'root_uuid')
             driver_int_info = task.node.driver_internal_info
             self.assertEqual('root_uuid',
                              driver_int_info['root_uuid_or_disk_id']),
-            uuid_mock.assert_called_once_with(task.driver.deploy,
-                                              task, 'root_uuid')
             boot_mode_mock.assert_called_once_with(task.node)
-
-    @mock.patch.object(agent.AgentDeployMixin, '_get_uuid_from_result',
-                       autospec=True)
-    @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
-    @mock.patch.object(fake.FakePower, 'get_power_state',
-                       spec=types.FunctionType)
-    @mock.patch.object(agent_client.AgentClient, 'power_off',
-                       spec=types.FunctionType)
-    @mock.patch.object(pxe.PXEBoot, 'prepare_instance',
-                       autospec=True)
-    @mock.patch('ironic.drivers.modules.agent.AgentDeployMixin'
-                '.check_deploy_success', autospec=True)
-    @mock.patch.object(pxe.PXEBoot, 'clean_up_ramdisk', autospec=True)
-    def test_reboot_to_instance_boot_none(self, clean_pxe_mock,
-                                          check_deploy_mock,
-                                          prepare_mock, power_off_mock,
-                                          get_power_state_mock,
-                                          node_power_action_mock,
-                                          uuid_mock):
-        check_deploy_mock.return_value = None
-        self.node.provision_state = states.DEPLOYWAIT
-        self.node.target_provision_state = states.ACTIVE
-        self.node.save()
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=False) as task:
-            get_power_state_mock.return_value = states.POWER_OFF
-            driver_internal_info = task.node.driver_internal_info
-            driver_internal_info['is_whole_disk_image'] = True
-            task.node.driver_internal_info = driver_internal_info
-            task.driver.boot = None
-
-            task.driver.deploy.reboot_to_instance(task)
-
-            self.assertFalse(clean_pxe_mock.called)
-            self.assertFalse(prepare_mock.called)
+            self.assertFalse(log_mock.called)
+            prepare_instance_mock.assert_called_once_with(mock.ANY, task,
+                                                          'root_uuid', None)
             power_off_mock.assert_called_once_with(task.node)
-            check_deploy_mock.assert_called_once_with(mock.ANY, task.node)
-            self.assertNotIn('root_uuid_or_disk_id',
-                             task.node.driver_internal_info)
-
             get_power_state_mock.assert_called_once_with(task)
             node_power_action_mock.assert_called_once_with(
                 task, states.POWER_ON)
             self.assertEqual(states.ACTIVE, task.node.provision_state)
             self.assertEqual(states.NOSTATE, task.node.target_provision_state)
-            self.assertFalse(uuid_mock.called)
 
+    @mock.patch.object(agent.LOG, 'warning', spec_set=True, autospec=True)
     @mock.patch.object(driver_utils, 'collect_ramdisk_logs', autospec=True)
     @mock.patch.object(agent.AgentDeployMixin, '_get_uuid_from_result',
                        autospec=True)
@@ -899,15 +1098,14 @@ class TestAgentDeploy(db_base.DbTestCase):
                        spec=types.FunctionType)
     @mock.patch.object(agent_client.AgentClient, 'power_off',
                        spec=types.FunctionType)
-    @mock.patch.object(pxe.PXEBoot, 'prepare_instance',
+    @mock.patch.object(agent.AgentDeployMixin, 'prepare_instance_to_boot',
                        autospec=True)
     @mock.patch('ironic.drivers.modules.agent.AgentDeployMixin'
                 '.check_deploy_success', autospec=True)
-    @mock.patch.object(pxe.PXEBoot, 'clean_up_ramdisk', autospec=True)
     def test_reboot_to_instance_boot_error(
-            self, clean_pxe_mock, check_deploy_mock, prepare_mock,
+            self, check_deploy_mock, prepare_instance_mock,
             power_off_mock, get_power_state_mock, node_power_action_mock,
-            uuid_mock, collect_ramdisk_logs_mock):
+            uuid_mock, collect_ramdisk_logs_mock, log_mock):
         check_deploy_mock.return_value = "Error"
         uuid_mock.return_value = None
         self.node.provision_state = states.DEPLOYWAIT
@@ -917,20 +1115,17 @@ class TestAgentDeploy(db_base.DbTestCase):
                                   shared=False) as task:
             get_power_state_mock.return_value = states.POWER_OFF
             task.node.driver_internal_info['is_whole_disk_image'] = True
-            task.driver.boot = None
             task.driver.deploy.reboot_to_instance(task)
-
-            self.assertFalse(clean_pxe_mock.called)
-            self.assertFalse(prepare_mock.called)
-            self.assertFalse(power_off_mock.called)
             check_deploy_mock.assert_called_once_with(mock.ANY, task.node)
+            self.assertFalse(prepare_instance_mock.called)
+            self.assertFalse(log_mock.called)
+            self.assertFalse(power_off_mock.called)
+            collect_ramdisk_logs_mock.assert_called_once_with(task.node)
             self.assertEqual(states.DEPLOYFAIL, task.node.provision_state)
             self.assertEqual(states.ACTIVE, task.node.target_provision_state)
-            collect_ramdisk_logs_mock.assert_called_once_with(task.node)
 
-    @mock.patch.object(agent_base_vendor.AgentDeployMixin,
-                       'configure_local_boot', autospec=True)
-    @mock.patch.object(deploy_utils, 'try_set_boot_device', autospec=True)
+    @mock.patch.object(agent.LOG, 'warning', spec_set=True, autospec=True)
+    @mock.patch.object(deploy_utils, 'get_boot_mode_for_deploy', autospec=True)
     @mock.patch.object(agent.AgentDeployMixin, '_get_uuid_from_result',
                        autospec=True)
     @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
@@ -938,19 +1133,17 @@ class TestAgentDeploy(db_base.DbTestCase):
                        spec=types.FunctionType)
     @mock.patch.object(agent_client.AgentClient, 'power_off',
                        spec=types.FunctionType)
-    @mock.patch.object(pxe.PXEBoot, 'prepare_instance',
+    @mock.patch.object(agent.AgentDeployMixin, 'prepare_instance_to_boot',
                        autospec=True)
     @mock.patch('ironic.drivers.modules.agent.AgentDeployMixin'
                 '.check_deploy_success', autospec=True)
-    @mock.patch.object(pxe.PXEBoot, 'clean_up_ramdisk', autospec=True)
-    def test_reboot_to_instance_localboot(self, clean_pxe_mock,
-                                          check_deploy_mock,
-                                          prepare_mock, power_off_mock,
+    def test_reboot_to_instance_localboot(self, check_deploy_mock,
+                                          prepare_instance_mock,
+                                          power_off_mock,
                                           get_power_state_mock,
                                           node_power_action_mock,
-                                          uuid_mock,
-                                          bootdev_mock,
-                                          configure_mock):
+                                          uuid_mock, boot_mode_mock,
+                                          log_mock):
         check_deploy_mock.return_value = None
         uuid_mock.side_effect = ['root_uuid', 'efi_uuid']
         self.node.provision_state = states.DEPLOYWAIT
@@ -965,11 +1158,21 @@ class TestAgentDeploy(db_base.DbTestCase):
             task.node.driver_internal_info = driver_internal_info
             boot_option = {'capabilities': '{"boot_option": "local"}'}
             task.node.instance_info = boot_option
+            boot_mode_mock.return_value = 'uefi'
             task.driver.deploy.reboot_to_instance(task)
 
-            self.assertFalse(clean_pxe_mock.called)
             check_deploy_mock.assert_called_once_with(mock.ANY, task.node)
-            self.assertFalse(bootdev_mock.called)
+            driver_int_info = task.node.driver_internal_info
+            self.assertEqual('root_uuid',
+                             driver_int_info['root_uuid_or_disk_id']),
+            uuid_mock_calls = [
+                mock.call(mock.ANY, task, 'root_uuid'),
+                mock.call(mock.ANY, task, 'efi_system_partition_uuid')]
+            uuid_mock.assert_has_calls(uuid_mock_calls)
+            boot_mode_mock.assert_called_once_with(task.node)
+            self.assertFalse(log_mock.called)
+            prepare_instance_mock.assert_called_once_with(
+                mock.ANY, task, 'root_uuid', 'efi_uuid')
             power_off_mock.assert_called_once_with(task.node)
             get_power_state_mock.assert_called_once_with(task)
             node_power_action_mock.assert_called_once_with(
